@@ -2,7 +2,9 @@
 
 from __future__ import annotations
 
+import logging
 import math
+import sys
 import time
 from collections import defaultdict
 from dataclasses import dataclass
@@ -63,11 +65,13 @@ class ProgressTable:
         self.header_printed = False
         self.last_header_printed_at_row_count = 0
         self.reprint_header_every_n_rows = reprint_header_every_n_rows
-        self.print_row_on_setitem = print_row_on_update
+        self.print_row_on_update = print_row_on_update
 
         self.custom_format = custom_format or self.get_default_format_func(num_decimal_places)
         self.needs_line_ending = False
         self.finished_rows: List[Dict[Any, Any]] = []
+        self.progress_bar_active = False
+        self.refresh_progress_bar: Callable = lambda: None
 
         for column in columns:
             self.add_column(column)
@@ -86,6 +90,10 @@ class ProgressTable:
 
     def add_column(self, name, width=None, color=None, aggregate=None):
         assert not self.header_printed, "Columns cannot be modified after printing the first row!"
+
+        if name in self.columns:
+            logging.warning(f"Column {name} already exists!")
+            self.columns.remove(name)
         self.columns.append(name)
 
         width = width if width is not None else self.default_width
@@ -103,6 +111,9 @@ class ProgressTable:
     def next_row(self, save=True):
         self._print_row()
         self._maybe_line_ending()
+        if self.progress_bar_active:
+            self.refresh_progress_bar()
+        sys.stdout.flush()
 
         if save and len(self.values) > 0:
             self.finished_rows.append(self.values)
@@ -112,7 +123,6 @@ class ProgressTable:
         self.aggregate_n = defaultdict(int)
 
     def close(self):
-        self._print_row()
         self.next_row()
         self._print_bottom_bar()
         self.header_printed = False
@@ -260,6 +270,7 @@ class ProgressTable:
         t_last_printed = -float("inf")
         t_beginning = time.time()
 
+        self.progress_bar_active = True
         for idx, element in enumerate(iterator):
             if time.time() - t_last_printed > 1 / self.progress_bar_fps:
                 print(end="\r")
@@ -272,11 +283,16 @@ class ProgressTable:
                 full_prefix.append("] ")
                 full_prefix = "".join(full_prefix)
                 full_prefix = full_prefix if full_prefix != " [] " else " "
-                self._print_progress_bar(idx, length, show_before=full_prefix)
+
+                self.refresh_progress_bar = lambda: self._print_progress_bar(
+                    idx, length, show_before=full_prefix
+                )
+                self.refresh_progress_bar()
 
                 t_last_printed = time.time()
-
             yield element
+
+        self.progress_bar_active = False
         self._print_row()
 
     def __setitem__(self, key, value):
@@ -296,5 +312,5 @@ class ProgressTable:
         else:
             self.values[key] = value
 
-        if self.print_row_on_setitem:
+        if self.print_row_on_update and not self.progress_bar_active:
             self._print_row()
